@@ -8,8 +8,18 @@ An implementation of a WebSocket connection.
 
 from collections import deque
 from enum import Enum
+from typing import Deque, Generator, List, Optional
 
-from .events import BytesMessage, CloseConnection, Message, Ping, Pong, TextMessage
+from .events import (
+    BytesMessage,
+    CloseConnection,
+    Event,
+    Message,
+    Ping,
+    Pong,
+    TextMessage,
+)
+from .extensions import Extension
 from .frame_protocol import CloseReason, FrameProtocol, Opcode, ParseFailed
 from .utilities import LocalProtocolError
 
@@ -19,15 +29,23 @@ class ConnectionState(Enum):
     RFC 6455, Section 4 - Opening Handshake
     """
 
+    #: The opening handshake is in progress.
     CONNECTING = 0
+    #: The opening handshake is complete.
     OPEN = 1
+    #: The remote WebSocket has initiated a connection close.
     REMOTE_CLOSING = 2
+    #: The local WebSocket (i.e. this instance) has initiated a connection close.
     LOCAL_CLOSING = 3
+    #: The closing handshake has completed.
     CLOSED = 4
+    #: The connection was rejected during the opening handshake.
     REJECTING = 5
 
 
 class ConnectionType(Enum):
+    """ An enumeration of connection types. """
+
     #: This connection will act as client and talk to a remote server
     CLIENT = 1
 
@@ -39,7 +57,7 @@ CLIENT = ConnectionType.CLIENT
 SERVER = ConnectionType.SERVER
 
 
-class Connection(object):
+class Connection:
     """
     A low-level WebSocket connection object.
 
@@ -53,21 +71,23 @@ class Connection(object):
     :type conn_type: ``ConnectionType``
     """
 
-    def __init__(self, connection_type, extensions=None, trailing_data=b""):
-        # type: (bool, Optional[List[Extension]], bytes) -> None
+    def __init__(
+        self,
+        connection_type: ConnectionType,
+        extensions: Optional[List[Extension]] = None,
+        trailing_data: bytes = b"",
+    ) -> None:
         self.client = connection_type is ConnectionType.CLIENT
-        self._events = deque()
+        self._events: Deque[Event] = deque()
         self._proto = FrameProtocol(self.client, extensions or [])
         self._state = ConnectionState.OPEN
         self.receive_data(trailing_data)
 
     @property
-    def state(self):
-        # type: () -> ConnectionState
+    def state(self) -> ConnectionState:
         return self._state
 
-    def send(self, event):
-        # type: (wsproto.events.Event) -> bytes
+    def send(self, event: Event) -> bytes:
         data = b""
         if isinstance(event, Message):
             data += self._proto.send_data(event.data, event.message_finished)
@@ -89,8 +109,7 @@ class Connection(object):
             raise LocalProtocolError("Event {} cannot be sent.".format(event))
         return data
 
-    def receive_data(self, data):
-        # type: (bytes) -> None
+    def receive_data(self, data: Optional[bytes]) -> None:
         """
         Pass some received data to the connection for handling.
 
@@ -115,8 +134,7 @@ class Connection(object):
         elif self.state is ConnectionState.CLOSED:
             raise LocalProtocolError("Connection already closed.")
 
-    def events(self):
-        # type: () -> Generator[Event, None, None]
+    def events(self) -> Generator[Event, None, None]:
         """
         Return a generator that provides any events that have been generated
         by protocol activity.
@@ -130,13 +148,16 @@ class Connection(object):
             for frame in self._proto.received_frames():
                 if frame.opcode is Opcode.PING:
                     assert frame.frame_finished and frame.message_finished
+                    assert isinstance(frame.payload, (bytes, bytearray))
                     yield Ping(payload=frame.payload)
 
                 elif frame.opcode is Opcode.PONG:
                     assert frame.frame_finished and frame.message_finished
+                    assert isinstance(frame.payload, (bytes, bytearray))
                     yield Pong(payload=frame.payload)
 
                 elif frame.opcode is Opcode.CLOSE:
+                    assert isinstance(frame.payload, tuple)
                     code, reason = frame.payload
                     if self.state is ConnectionState.LOCAL_CLOSING:
                         self._state = ConnectionState.CLOSED
@@ -145,6 +166,7 @@ class Connection(object):
                     yield CloseConnection(code=code, reason=reason)
 
                 elif frame.opcode is Opcode.TEXT:
+                    assert isinstance(frame.payload, str)
                     yield TextMessage(
                         data=frame.payload,
                         frame_finished=frame.frame_finished,
@@ -152,6 +174,7 @@ class Connection(object):
                     )
 
                 elif frame.opcode is Opcode.BINARY:
+                    assert isinstance(frame.payload, (bytes, bytearray))
                     yield BytesMessage(
                         data=frame.payload,
                         frame_finished=frame.frame_finished,
